# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['ha_franklin']

package_data = \
{'': ['*']}

install_requires = \
['ha-mqtt-discoverable>=0.8.1,<0.9.0',
 'pyaml>=21.10.1,<22.0.0',
 'shell>=1.0.1,<2.0.0']

entry_points = \
{'console_scripts': ['ha-cupsd-check-printer = '
                     'ha_franklin.cli:check_cupsd_single_queue_status_app',
                     'ha-cupsd-info = ha_franklin.cli:app_summary',
                     'ha-cupsd-monitor = '
                     'ha_franklin.cli:monitor_cupsd_queue_app',
                     'ha-cupsd-monitor-queues = '
                     'ha_franklin.cli:monitor_cupsd_queue_app',
                     'ha-cupsd-monitor-single-queue = '
                     'ha_franklin.cli:monitor_single_cupsd_queue_app',
                     'ha-cupsd-monitor-version = ha_franklin.cli:app_version',
                     'ha-cupsd-version = ha_franklin.cli:app_version']}

setup_kwargs = {
    'name': 'ha-franklin',
    'version': '0.1.0',
    'description': 'ha-franklin monitors CUPSD queues and writes information to MQTT for Home Assistant',
    'long_description': '\n# ha-franklin\n\n[![License](https://img.shields.io/github/license/unixorn/ha-franklin.svg)](https://opensource.org/license/apache-2-0/)\n[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)\n[![GitHub last commit (branch)](https://img.shields.io/github/last-commit/unixorn/ha-franklin/main.svg)](https://github.com/unixorn/ha-franklin)\n[![Downloads](https://static.pepy.tech/badge/ha-franklin)](https://pepy.tech/project/ha-franklin)\n\n<!-- START doctoc generated TOC please keep comment here to allow auto update -->\n<!-- DON\'T EDIT THIS SECTION, INSTEAD RE-RUN doctoc TO UPDATE -->\n## Table of Contents\n\n- [Background](#background)\n- [Usage](#usage)\n  - [Configuration](#configuration)\n  - [Running the Monitor](#running-the-monitor)\n\n<!-- END doctoc generated TOC please keep comment here to allow auto update -->\n\n## Background\n\nI wanted a non-toy test example of using [ha-mqtt-discoverable](https://github.com/unixorn/ha-mqtt-discoverable/tree/v0.8.1).\n\n`ha-franklin` will monitor CUPSD print queues, and present a binary sensor to Home Assistant over MQTT showing whether the printer is printing.\n\nI use this to turn the smart switch for the HP 4050N in the basement on and off so that by the time I walk downstairs from my office after printing something, Home Assistant has turned on the power to the printer and the job has started printing.\n\n\n## Usage\n\n### Configuration\n\nCreate a config file (yaml) with a list of dictionaries in it. Each dictionary should have the following keys:\n- `mqtt_server`: DNS name or a raw IP.\n- `mqtt_user`: the_mqtt_user\n- `mqtt_password`: the_mqtt_password\n- `name`: Franklin@cupsd\n- `unique_id`: printername-cupsd\n- `cupsd_queue_name`: Queue_name_on_cupsd_server\n- `cupsd_server`: cupsd.example.com\n- `check_interval`: 10\n\nThe easiest way to create a configuration file is to start by copying `config/config-example.yaml` and editing it to fit.\n\n### Running the Monitor\n\nI recommend using `docker`, `nerdctl` or `podman` to run the tooling in a container.\n\n`docker run -v "$(pwd)/config":/config --rm unixorn/ha-franklin ha-cupsd-monitor-queues --settings-file /config/config.yaml`\n',
    'author': 'Joe Block',
    'author_email': 'jpb@unixorn.net',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.10.0,<4.0',
}


setup(**setup_kwargs)



#    Copyright 2023 Joe Block <jpb@unixorn.net>
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import logging
import os
import yaml


def read_yaml_file(path: str):
    """
    Return the data structure contained in a yaml file

    Args:
        path (str): Path to read from

    Returns:
        Data decoded from YAML file content
    """
    with open(path) as yamlFile:
        data = yaml.safe_load(yamlFile)
        return data


def write_yaml_file(path: str, data):
    """
    Writes a data structure into a YAML file

    Args:
        path (str): Path to data file
        data (any): Data to convert and write
    """
    with open(path, "w") as yamlFile:
        yaml.safe_dump(data, yamlFile)


def valid_settings(settings: dict = {}) -> bool:
    """
    Validate that we have all settings we need

    Args:
        settings: Dictionary of settings

    Returns:
        bool of validity
    """
    fails = []
    required = (
        "cupsd_server",
        "cupsd_queue_name",
        "check_interval",
        "mqtt_server",
        "mqtt_user",
        "mqtt_password",
    )
    logging.debug(f"Settings: {settings}")
    for rs in required:
        logging.debug(f"Checking for {rs}...")
        if rs not in settings:
            fail_message = f"'{rs}' missing from settings."
            fails.append(fail_message)
            logging.critical(fail_message)
    if len(fails) > 0:
        return False
    else:
        return True


def load_monitor_settings(path: str, cli):
    """
    Load settings from a yaml file, allowing overrides from the CLI

    Args:
        path: Path to configuration file
        cli (argparse cli object): Command line options

    Returns:
        dict: A dictionary containing all of our settings
    """
    settings = {}
    # Allow overrides from command line
    if cli.cupsd_server:
        settings["cupsd_server"] = cli.cupsd_server
    if cli.cupsd_queue_name:
        settings["cupsd_queue_name"] = cli.cupsd_queue_name
    if cli.check_interval:
        settings["check_interval"] = cli.check_interval
    if cli.mqtt_server:
        settings["mqtt_server"] = cli.mqtt_server
    if cli.mqtt_password:
        settings["mqtt_password"] = cli.mqtt_password
    if cli.mqtt_user:
        settings["mqtt_user"] = cli.mqtt_user

    logging.debug(f"Processed settings: {settings}")
    return settings


def load_multiple_monitor_settings(path: str, cli):
    """
    Load settings from a yaml file

    Args:
        path: Path to configuration file
        cli (argparse cli object): Command line options

    Returns:
        dict: A dictionary containing settings for a group of CUPSD queues
    """
    if os.access(path, os.R_OK):
        settings = read_yaml_file(path=path)
    else:
        logging.error(f"Could not read {path}")
        settings = {}
    logging.debug(f"Base settings: {settings}")

    logging.debug(f"Processed settings: {settings}")
    return settings


def setup_logging(
    log_level: str = "INFO",
    log_format: str = "[%(asctime)s][%(levelname)8s][%(filename)s:%(lineno)s - %(funcName)20s() ] %(message)s",
):
    """
    Set up logging

    Args:
        cli - An argparse cli object
    """
    log_level = getattr(logging, log_level.upper(), None)
    logging.basicConfig(level=log_level, format=log_format)
    logging.info("Log level set to %s", log_level)
    logging.debug(f"Using '{log_format}' for log format")
